<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <link href="https://cdn.jsdelivr.net/npm/bootstrap@5.2.0-beta1/dist/css/bootstrap.min.css" rel="stylesheet">
    <title>Security Baseline Assessment Report</title>
</head>
<body>
    <svg xmlns="http://www.w3.org/2000/svg" style="display: none;">
        <symbol id="check-circle-fill" fill="currentColor" viewBox="0 0 16 16">
            <path
                d="M16 8A8 8 0 1 1 0 8a8 8 0 0 1 16 0zm-3.97-3.03a.75.75 0 0 0-1.08.022L7.477 9.417 5.384 7.323a.75.75 0 0 0-1.06 1.06L6.97 11.03a.75.75 0 0 0 1.079-.02l3.992-4.99a.75.75 0 0 0-.01-1.05z">
            </path>
        </symbol>
        <symbol id="info-fill" fill="currentColor" viewBox="0 0 16 16">
            <path
                d="M8 16A8 8 0 1 0 8 0a8 8 0 0 0 0 16zm.93-9.412-1 4.705c-.07.34.029.533.304.533.194 0 .487-.07.686-.246l-.088.416c-.287.346-.92.598-1.465.598-.703 0-1.002-.422-.808-1.319l.738-3.468c.064-.293.006-.399-.287-.47l-.451-.081.082-.381 2.29-.287zM8 5.5a1 1 0 1 1 0-2 1 1 0 0 1 0 2z">
            </path>
        </symbol>
        <symbol id="exclamation-triangle-fill" fill="currentColor" viewBox="0 0 16 16">
            <path
                d="M8.982 1.566a1.13 1.13 0 0 0-1.96 0L.165 13.233c-.457.778.091 1.767.98 1.767h13.713c.889 0 1.438-.99.98-1.767L8.982 1.566zM8 5c.535 0 .954.462.9.995l-.35 3.507a.552.552 0 0 1-1.1 0L7.1 5.995A.905.905 0 0 1 8 5zm.002 6a1 1 0 1 1 0 2 1 1 0 0 1 0-2z">
            </path>
        </symbol>
        <symbol id="bi-x-circle-fill" fill="currentColor" viewBox="0 0 16 16">
            <path
                d="M16 8A8 8 0 1 1 0 8a8 8 0 0 1 16 0zM5.354 4.646a.5.5 0 1 0-.708.708L7.293 8l-2.647 2.646a.5.5 0 0 0 .708.708L8 8.707l2.646 2.647a.5.5 0 0 0 .708-.708L8.707 8l2.647-2.646a.5.5 0 0 0-.708-.708L8 7.293 5.354 4.646z">
            </path>
        </symbol>
        <symbol id="bi-dash-circle" fill="currentColor" viewBox="0 0 16 16">
            <path
                d="M8 15A7 7 0 1 1 8 1a7 7 0 0 1 0 14zm0 1A8 8 0 1 0 8 0a8 8 0 0 0 0 16z">
            </path>
            <path
                d="M4 8a.5.5 0 0 1 .5-.5h7a.5.5 0 0 1 0 1h-7A.5.5 0 0 1 4 8z">
            </path>
        </symbol>
    </svg>

    <div class="container">
    <div class="row justify-content-center">
        <div class="col-10 col-lg-8">
            <br>
            <h1 class="text-center">Security Baseline Assessment Report</h1>
            <br>
            <div class="row justify-content-center">
                <div class="col-6"></div>
                <div class="col-6">
                    <table class="table">
                        <tr>
                            <td>AWS Account</td><td>{{ account_id }}</td>
                        </tr>
                        <tr>
                            <td>Generated at</td><td>{{ generated_at }}</td>
                        </tr>
                    </table>
                </div>
            </div>

            <div class="row justify-content-center">
            <div class="card">
                <div class="card-body">
                    <div class="card-title">
                        <b>Overview</b>
                    </div>

                    <div class="row">
                        {% for level, count in overview %}
                        <div class="col">
                            <div class="alert alert-{% if level == 'Error' %}dark{% elif level == 'Info' %}primary{% else %}{{ level|lower }}{% endif %}">
                                <div class="d-flex align-items-center justify-content-around">
                                    <h1><a href="#{{ level|lower }}-list" class="link-{% if level == 'Error' %}secondary{% elif level == 'Info' %}primary{% else %}{{ level|lower }}{% endif %}">{{ count }}</a></h1>
                                </div>
                                <div class="row align-items-center justify-content-center h6">{{ level|upper }}</div>
                            </div>
                        </div>
                        {% endfor %}
                    </div>
                </div>
            </div>
            </div>

            <br>

            <div class="row justify-content-center">
            <div class="card">
                {% for section in result_sections %}
                <div class="card-body" id="{{ section.level|lower }}-list">
                <div class="accordion" id="accordionSection{{ section.level }}">

                    {% for item in section.result_items %}
                    <div class="accordion-item">
                    <h2 class="accordion-header" id="panelsStayOpen-heading-{{ section.level|lower }}-{{ '%02d'|format(loop.index) }}">
                        <button class="accordion-button collapsed" type="button" data-bs-toggle="collapse" data-bs-target="#panelsStayOpen-collapse-{{ section.level|lower }}-{{ '%02d'|format(loop.index)}}" aria-expanded="true" aria-controls="panelsStayOpen-collapse-{{ section.level|lower }}-{{ '%02d'|format(loop.index)}}">
                            <span class="badge text-bg-{% if section.level == 'Error' %}dark{% elif section.level == 'Info' %}primary{% else %}{{ section.level|lower }}{% endif %} rounded-pill">{{ section.level }}</span>
                            <div class="ms-2 me-auto">{{ item.title }}</div>
                        </button>
                    </h2>
                    <div id="panelsStayOpen-collapse-{{ section.level|lower }}-{{ '%02d'|format(loop.index)}}" class="accordion-collapse collapse" aria-labelledby="panelsStayOpen-heading-{{ section.level|lower }}-{{ '%02d'|format(loop.index) }}">
                        <div class="accordion-body">
                            <div class="alert alert-{% if section.level == 'Error' %}dark{% elif section.level == 'Info' %}primary{% else %}{{ section.level|lower }}{% endif %} d-flex align-items-center" role="alert">
                                <svg aria-label="{{ section.level }}:" class="bi flex-shrink-0 me-2" width="24" height="24" role="img">
                                    <use xlink:href="{% if section.level == 'Success' %}#check-circle-fill{% elif section.level == 'Info' or 'Error' %}#info-fill{% else %}#exclamation-triangle-fill{% endif %}"></use>
                                </svg>
                                <span>{{ item.message }}</span>
                            </div>
                            <table class="table">
                                <thead>
                                    <tr>
                                    {% for header in item.table.headers %}
                                    <th>{{ header }}</th>
                                    {% endfor %}
                                    </tr>
                                </thead>
                                <tbody>
                                {% for row in item.table.rows %}
                                <tr>
                                    {% for cell in row %}
                                    <td>{{ cell }}</td>
                                    {% endfor %}
                                </tr>
                                {% endfor %}
                                </tbody>
                            </table>
                        </div>
                    </div>
                    </div>
                    {% endfor %}
                </div>
                </div>
                {% endfor %}
            </div>
            </div>
        </div>
    </div>
    </div>

    <div class="container">
        <div class="row justify-content-center">
        <div class="col-10 col-lg-8">
        <br>

            <!-- Below this section, you can add content references for each language. -->

        </div>
        </div>
    </div>

    <script src="https://cdn.jsdelivr.net/npm/bootstrap@5.2.0-beta1/dist/js/bootstrap.bundle.min.js"></script>
</body>
</html>
