#!/usr/bin/env python3
"""
Runbooks CLI Entry Point

This module enables the package to be executed as:
    python -m runbooks
    uv run python -m runbooks
"""

from runbooks.main import main

if __name__ == "__main__":
    main()
