from enum import Enum


class CREDENTIAL_REPORT_COLS(Enum):
    USER = 0
    ARN = 1
    USER_CREATION_TIME = 2
    PASSWORD_ENABLED = 3
    PASSWORD_LAST_USED = 4
    PASSWORD_LAST_CHANGED = 5
    PASSWORD_NEXT_ROTATION = 6
    MFA_ACTIVE = 7
    ACCESS_KEY_1_ACTIVE = 8
    ACCESS_KEY_1_LAST_ROTATED = 9
    ACCESS_KEY_1_LAST_USED_DATE = 10
    ACCESS_KEY_1_LAST_USED_REGION = 11
    ACCESS_KEY_1_LAST_USED_SERVICE = 12
    ACCESS_KEY_2_ACTIVE = 13
    ACCESS_KEY_2_LAST_ROTATED = 14
    ACCESS_KEY_2_LAST_USED_DATE = 15
    ACCESS_KEY_2_LAST_USED_REGION = 16
    ACCESS_KEY_2_LAST_USED_SERVICE = 17
    CERT_1_ACTIVE = 18
    CERT_1_LAST_ROTATED = 19
    CERT_2_ACTIVE = 20
    CERT_2_LAST_ROTATED = 21


class CHECKLIST_INDEX_CONST(Enum):
    ROOT_MFA_SETTING_CHECK = 0
    ROOT_USAGE_CHECK = 1
    ROOT_CREDENTIAL_CHECK = 2
    IAM_MFA_SETTING_CHECK = 3
    IAM_PASSWORD_POLICY_CHECK = 4
    NON_GROUP_POLICY_CHECK = 5
    ALTERNATE_CONTACT_CHECK = 6
    TRAIL_ENABLED_CHECK = 7
    MULTIREGION_TRAIL_CHECK = 8
    ACCOUNT_LEVEL_BUCKET_PUBLIC_ACCESS_CHECK = 9
    BUCKET_LEVEL_PUBLIC_ACCESS_CHECK = 10
    CLOUDWATCH_ALARM_CONFIGURATION_CHECK = 11
    MULTIREGION_INSTANCE_USAGE_CHECK = 12
    GUARD_DUTY_ENABLED_CHECK = 13
    TRUST_ADVISOR_CHECK = 14
